@@ -11,7 +11,6 @@ from typing import (
     TYPE_CHECKING,
     Any,
     AsyncIterable,
-    Awaitable,
     Callable,
     Coroutine,
     Dict,
@@ -75,7 +74,6 @@ from meadowrun.run_job_core import (
     CloudProviderType,
     GridJobCloudInterface,
     JobCompletion,
-    ObjectStorage,
     SshHost,
     WaitOption,
 )
@@ -84,12 +82,15 @@ from meadowrun.s3_grid_job import receive_results
 if TYPE_CHECKING:
     from types import TracebackType
 
-    from meadowrun.meadowrun_pb2 import Job, ProcessState
-    from meadowrun.run_job_local import AgentTaskWorkerServer, Stats
+    from meadowrun.meadowrun_pb2 import Job
+    from meadowrun.object_storage import ObjectStorage
+    from meadowrun.run_job_core import TaskProcessState, WorkerProcessState
+    from meadowrun.run_job_local import TaskWorkerServer, WorkerMonitor
     from types_aiobotocore_s3 import S3Client
     from types_aiobotocore_sqs import SQSClient
     from typing_extensions import Literal
 
+
 # SEE ALSO ec2_alloc_stub.py
 
 _T = TypeVar("_T")
@@ -99,26 +100,26 @@ _U = TypeVar("_U")
 # replicate into each region.
 _AMIS = {
     "plain": {
-        "us-east-2": "ami-07f0f36fa29c65aaa",
-        "us-east-1": "ami-0ee6b624d61b9b641",
-        "us-west-1": "ami-06f0dd65aff3c563b",
-        "us-west-2": "ami-02d0917b63f6b9bae",
-        "eu-central-1": "ami-0e77a4d360d7b7610",
-        "eu-west-1": "ami-040289dd65de37dda",
-        "eu-west-2": "ami-03c5174fcd071728e",
-        "eu-west-3": "ami-0af288ea3afdeb8a6",
-        "eu-north-1": "ami-0cdd55e01ba94dbdd",
+        "us-east-2": "ami-02cb932ead54f2c81",
+        "us-east-1": "ami-0c4f30064b16cac99",
+        "us-west-1": "ami-012fbb53effdd0f6f",
+        "us-west-2": "ami-039fa3adc59d9335b",
+        "eu-central-1": "ami-0f920323773495495",
+        "eu-west-1": "ami-037a59c4c352cfe48",
+        "eu-west-2": "ami-0e768a7fdd3b895fe",
+        "eu-west-3": "ami-0f46081510e4539ff",
+        "eu-north-1": "ami-02d2946f47e774859",
     },
     "cuda": {
-        "us-east-2": "ami-0ab0964aab023b6b2",
-        "us-east-1": "ami-0e9c845879ca81e49",
-        "us-west-1": "ami-00cbbc06817d02c38",
-        "us-west-2": "ami-00e359519a1e690e6",
-        "eu-central-1": "ami-06ec989f961db4c5c",
-        "eu-west-1": "ami-0bd8d3622c2af4e82",
-        "eu-west-2": "ami-0b88f83ef01a49dd8",
-        "eu-west-3": "ami-061cbd409324c685d",
-        "eu-north-1": "ami-030f6e8762e7b6220",
+        "us-east-2": "ami-03c47749b6700ef26",
+        "us-east-1": "ami-01884f83d0fd5cad4",
+        "us-west-1": "ami-0ce1ef3db8dd5cb59",
+        "us-west-2": "ami-08f11c1824838d2c6",
+        "eu-central-1": "ami-01e7286e8f0e33e45",
+        "eu-west-1": "ami-08d3ed38be713e3c2",
+        "eu-west-2": "ami-0f0eaaa2947ac76ce",
+        "eu-west-3": "ami-059fc114d6efb01bf",
+        "eu-north-1": "ami-09279ba9a41266369",
     },
 }
 SSH_USER = "ubuntu"
@@ -818,7 +819,7 @@ class EC2GridJobInterface(GridJobCloudInterface):
     async def get_worker_function(
         self, queue_index: int
     ) -> Callable[
-        [str, str, AgentTaskWorkerServer, Callable[[], Awaitable[Stats]]],
+        [str, str, TaskWorkerServer, WorkerMonitor],
         Coroutine[Any, Any, None],
     ]:
         if len(self._request_queue_urls) < queue_index + 1:
@@ -833,7 +834,7 @@ class EC2GridJobInterface(GridJobCloudInterface):
 
     async def receive_task_results(
         self, *, stop_receiving: asyncio.Event, workers_done: asyncio.Event
-    ) -> AsyncIterable[List[Tuple[int, int, ProcessState]]]:
+    ) -> AsyncIterable[Tuple[List[TaskProcessState], List[WorkerProcessState]]]:
         if self._s3_client is None:
             raise ValueError("EC2GridJobInterface must be created with `async with`")
 
