@@ -1,17 +1,13 @@
 from __future__ import annotations
 
-import asyncio
 import itertools
 import json
 import os
-import pickle
 import time
 import traceback
 from typing import (
     TYPE_CHECKING,
     Any,
-    Awaitable,
-    Callable,
     Iterable,
     List,
     Optional,
@@ -28,12 +24,12 @@ from meadowrun.aws_integration.management_lambdas.ec2_alloc_stub import (
 from meadowrun.aws_integration.s3 import _get_bucket_name
 from meadowrun.meadowrun_pb2 import ProcessState
 from meadowrun.s3_grid_job import upload_task_args, download_task_arg, complete_task
-from meadowrun.run_job_local import StatsAccumulator, _accumulate_stats
+from meadowrun.run_job_local import restart_worker
 
 if TYPE_CHECKING:
     from types_aiobotocore_s3.client import S3Client
     from types_aiobotocore_sqs.client import SQSClient
-    from meadowrun.run_job_local import AgentTaskWorkerServer, Stats
+    from meadowrun.run_job_local import TaskWorkerServer, WorkerMonitor
 
 _REQUEST_QUEUE_NAME_PREFIX = "meadowrun-task-"
 
@@ -134,15 +130,15 @@ async def retry_task(
     request_queue_url: str,
     task_id: int,
     attempt: int,
-    range: Tuple[int, int],
+    byte_range: Tuple[int, int],
     sqs_client: SQSClient,
 ) -> None:
     task = json.dumps(
         dict(
             task_id=task_id,
             attempt=attempt,
-            range_from=range[0],
-            range_end=range[1],
+            range_from=byte_range[0],
+            range_end=byte_range[1],
         )
     )
 
@@ -255,8 +251,8 @@ async def _worker_iteration(
     region_name: str,
     log_file_name: str,
     pid: int,
-    worker_server: AgentTaskWorkerServer,
-    get_stats: Callable[[], Awaitable[Stats]],
+    worker_server: TaskWorkerServer,
+    worker_monitor: WorkerMonitor,
 ) -> bool:
     task = await _get_task(
         sqs,
@@ -271,54 +267,48 @@ async def _worker_iteration(
         return False
 
     task_id, attempt, arg = task
-    cont = True
     print(f"Meadowrun agent: About to execute task #{task_id}, attempt #{attempt}")
-    stats = StatsAccumulator()
     try:
+        worker_monitor.start_stats()
         await worker_server.send_message(arg)
 
-        accumulate_stats_task = asyncio.create_task(_accumulate_stats(get_stats, stats))
-        receive_message_task = asyncio.create_task(worker_server.receive_message())
-
-        await asyncio.wait(
-            [receive_message_task, accumulate_stats_task],
-            return_when=asyncio.FIRST_COMPLETED,
-        )
-        state, result = await receive_message_task
-        accumulate_stats_task.cancel()
+        state, result = await worker_server.receive_message()
+        stats = await worker_monitor.stop_stats()
 
         process_state = ProcessState(
             state=ProcessState.ProcessStateEnum.SUCCEEDED
             if state == "SUCCEEDED"
             else ProcessState.ProcessStateEnum.PYTHON_EXCEPTION,
             pid=pid,
-            pickled_result=pickle.dumps(result, protocol=pickle.HIGHEST_PROTOCOL),
+            pickled_result=result,
             return_code=0,
             log_file_name=log_file_name,
             max_memory_used_gb=stats.max_memory_used_gb,
         )
 
-        print(
-            f"Meadowrun agent: Completed task #{task_id}, attempt #{attempt}, "
-            f"state {ProcessState.ProcessStateEnum.Name(process_state.state)}, max "
-            f"memory {stats.max_memory_used_gb}GB "
-        )
     except Exception:
-        print(
-            f"Meadowrun agent: Unexpected task worker exit #{task_id}, attempt "
-            f"#{attempt}, max memory {stats.max_memory_used_gb}GB: "
-            f"{traceback.format_exc()}"
-        )
+        stats = await worker_monitor.stop_stats()
 
         process_state = ProcessState(
             state=ProcessState.ProcessStateEnum.UNEXPECTED_WORKER_EXIT,
             pid=pid,
-            return_code=0,
+            return_code=(await worker_monitor.try_get_return_code()) or 0,
             log_file_name=log_file_name,
             max_memory_used_gb=stats.max_memory_used_gb,
         )
 
-        cont = False
+        print(
+            f"Meadowrun agent: Unexpected worker exit, restarting worker. "
+            f"In task #{task_id}, attempt #{attempt}, {traceback.format_exc()}"
+        )
+
+        await restart_worker(worker_server, worker_monitor)
+
+    print(
+        f"Meadowrun agent: Completed task #{task_id}, attempt #{attempt}, "
+        f"state {ProcessState.ProcessStateEnum.Name(process_state.state)}, max "
+        f"memory {process_state.max_memory_used_gb}GB "
+    )
 
     await complete_task(
         s3c,
@@ -329,7 +319,7 @@ async def _worker_iteration(
         process_state,
     )
 
-    return cont
+    return True
 
 
 async def worker_function(
@@ -338,8 +328,8 @@ async def worker_function(
     region_name: str,
     public_address: str,
     log_file_name: str,
-    worker_server: AgentTaskWorkerServer,
-    get_worker_stats: Callable[[], Awaitable[Stats]],
+    worker_server: TaskWorkerServer,
+    worker_monitor: WorkerMonitor,
 ) -> None:
     """
     Runs a loop that gets tasks off of the request_queue, communicates that via reader
@@ -361,6 +351,6 @@ async def worker_function(
             log_file_name,
             pid,
             worker_server,
-            get_worker_stats,
+            worker_monitor,
         ):
             pass
