@@ -17,10 +17,15 @@ from meadowrun.aws_integration.management_lambdas.ec2_alloc_stub import (
     _RUNNING_JOBS,
     ignore_boto3_error_code,
 )
+from meadowrun.aws_integration.management_lambdas.config import (
+    TERMINATE_INSTANCES_IF_IDLE_FOR_SECS,
+)
 
 
-# Terminate instances if they haven't run any jobs in the last 5 minutes
-_TERMINATE_INSTANCES_IF_IDLE_FOR = datetime.timedelta(minutes=5)
+# Terminate instances if they haven't run any jobs recently
+_TERMINATE_INSTANCES_IF_IDLE_FOR = datetime.timedelta(
+    seconds=TERMINATE_INSTANCES_IF_IDLE_FOR_SECS
+)
 # If we see instances running that aren't registered, we assume there is something wrong
 # and they need to be terminated. However, it's possible that we happen to query between
 # when an instance is launched and when it's registered. So for the first 30 seconds
@@ -61,7 +66,9 @@ def _get_registered_ec2_instances(
 
     return {
         item[_INSTANCE_ID]: (
-            datetime.datetime.fromisoformat(item[_LAST_UPDATE_TIME]),
+            datetime.datetime.fromisoformat(item[_LAST_UPDATE_TIME]).replace(
+                tzinfo=datetime.timezone.utc
+            ),
             len(item[_RUNNING_JOBS]),
         )
         for item in response["Items"]
@@ -150,8 +157,7 @@ def _deregister_and_terminate_instances(
 
     registered_instances = _get_registered_ec2_instances(region_name)
 
-    now = datetime.datetime.utcnow()
-    now_with_timezone = datetime.datetime.now(datetime.timezone.utc)
+    now = datetime.datetime.now(datetime.timezone.utc)
 
     for instance_id, (last_updated, num_jobs) in registered_instances.items():
         if instance_id not in running_instances:
@@ -176,7 +182,7 @@ def _deregister_and_terminate_instances(
 
     for instance_id, instance in non_terminated_instances.items():
         if instance_id not in registered_instances:
-            if (now_with_timezone - instance.launch_time) > launch_register_delay:
+            if (now - instance.launch_time) > launch_register_delay:
                 print(
                     f"{instance_id} is running but is not registered, will terminate. "
                     f"Was launched at {instance.launch_time}."
