@@ -7,9 +7,9 @@ import functools
 import json
 import uuid
 from typing import (
+    TYPE_CHECKING,
     Any,
     AsyncIterable,
-    Awaitable,
     Callable,
     Coroutine,
     Generic,
@@ -17,7 +17,6 @@ from typing import (
     List,
     Optional,
     Sequence,
-    TYPE_CHECKING,
     Tuple,
     Type,
     TypeVar,
@@ -35,9 +34,9 @@ from meadowrun.azure_integration.grid_tasks_queue import (
     add_tasks,
     add_worker_shutdown_message,
     create_queues_for_job,
-    retry_task,
     get_results_unordered,
-    worker_function_async,
+    retry_task,
+    worker_function,
 )
 from meadowrun.azure_integration.mgmt_functions.azure_constants import (
     ALLOCATED_TIME,
@@ -74,17 +73,19 @@ from meadowrun.run_job_core import (
     CloudProviderType,
     GridJobCloudInterface,
     JobCompletion,
-    ObjectStorage,
     SshHost,
     WaitOption,
 )
 
 if TYPE_CHECKING:
-    import asyncssh
-    from typing_extensions import Literal
     from types import TracebackType
-    from meadowrun.meadowrun_pb2 import Job, ProcessState
-    from meadowrun.run_job_local import AgentTaskWorkerServer, Stats
+
+    import asyncssh
+    from meadowrun.meadowrun_pb2 import Job
+    from meadowrun.object_storage import ObjectStorage
+    from meadowrun.run_job_core import TaskProcessState, WorkerProcessState
+    from meadowrun.run_job_local import TaskWorkerServer, WorkerMonitor
+    from typing_extensions import Literal
 
 
 _T = TypeVar("_T")
@@ -601,7 +602,7 @@ class AzureVMGridJobInterface(GridJobCloudInterface, Generic[_T, _U]):
     async def get_worker_function(
         self, queue_index: int
     ) -> Callable[
-        [str, str, AgentTaskWorkerServer, Callable[[], Awaitable[Stats]]],
+        [str, str, TaskWorkerServer, WorkerMonitor],
         Coroutine[Any, Any, None],
     ]:
         if self._request_result_queues is None:
@@ -616,15 +617,14 @@ class AzureVMGridJobInterface(GridJobCloudInterface, Generic[_T, _U]):
         request_queue, result_queue = await self._request_result_queues
 
         return functools.partial(
-            worker_function_async,
-            # user_function,
+            worker_function,
             request_queue,
             result_queue,
         )
 
     async def receive_task_results(
         self, *, stop_receiving: asyncio.Event, workers_done: asyncio.Event
-    ) -> AsyncIterable[List[Tuple[int, int, ProcessState]]]:
+    ) -> AsyncIterable[Tuple[List[TaskProcessState], List[WorkerProcessState]]]:
         if self._request_result_queues is None:
             raise ValueError(
                 "Must call setup_and_add_tasks before calling receive_task_results"
