@@ -12,8 +12,6 @@ from typing import (
     TYPE_CHECKING,
     Any,
     AsyncIterable,
-    Awaitable,
-    Callable,
     Iterable,
     List,
     Optional,
@@ -42,10 +40,11 @@ from meadowrun.azure_integration.mgmt_functions.azure_core.azure_storage_api imp
     queue_send_message,
 )
 from meadowrun.meadowrun_pb2 import GridTask, GridTaskStateResponse, ProcessState
+from meadowrun.run_job_core import TaskProcessState
+from meadowrun.run_job_local import WorkerMonitor, restart_worker
 
 if TYPE_CHECKING:
-    from meadowrun.run_job_core import AgentTaskWorkerServer
-    from meadowrun.run_job_local import Stats
+    from meadowrun.run_job_core import TaskWorkerServer, WorkerProcessState
 
 _T = TypeVar("_T")
 _U = TypeVar("_U")
@@ -115,7 +114,7 @@ async def retry_task(
         GridTask(
             task_id=task_id,
             attempt=attempt,
-            pickled_function_arguments=pickle.dumps(task),
+            pickled_function_arguments=pickle.dumps(((task,), {})),
         ).SerializeToString(),
     )
 
@@ -200,69 +199,77 @@ async def _worker_iteration(
     result_queue: Queue,
     log_file_name: str,
     pid: int,
-    worker_server: AgentTaskWorkerServer,
+    worker_server: TaskWorkerServer,
+    worker_monitor: WorkerMonitor,
 ) -> bool:
     task = await _get_task(request_queue, result_queue)
     if not task:
         print("Meadowrun agent: Received shutdown message. Exiting.")
         return False
 
-    cont = True
     print(
         f"Meadowrun agent: About to execute task #{task.task_id}, attempt "
         f"#{task.attempt}"
     )
 
     try:
+        worker_monitor.start_stats()
         await worker_server.send_message(task.pickled_function_arguments)
         state, result = await worker_server.receive_message()
+        stats = await worker_monitor.stop_stats()
 
         process_state = ProcessState(
             state=ProcessState.ProcessStateEnum.SUCCEEDED
             if state == "SUCCEEDED"
             else ProcessState.ProcessStateEnum.PYTHON_EXCEPTION,
             pid=pid,
-            pickled_result=pickle.dumps(result, protocol=pickle.HIGHEST_PROTOCOL),
+            pickled_result=result,
             return_code=0,
             log_file_name=log_file_name,
+            max_memory_used_gb=stats.max_memory_used_gb,
         )
 
-        print(
-            f"Meadowrun agent: Completed task #{task.task_id}, attempt {task.attempt}, "
-            f"state {ProcessState.ProcessStateEnum.Name(process_state.state)}"
-        )
     except Exception:
-        print(
-            f"Meadowrun agent: Unexpected task worker exit #{task.task_id}, attempt "
-            f"#{task.attempt}: {traceback.format_exc()}"
-        )
+        stats = await worker_monitor.stop_stats()
 
         process_state = ProcessState(
             state=ProcessState.ProcessStateEnum.UNEXPECTED_WORKER_EXIT,
             pid=pid,
-            return_code=0,
+            return_code=(await worker_monitor.try_get_return_code()) or 0,
             log_file_name=log_file_name,
+            max_memory_used_gb=stats.max_memory_used_gb,
         )
 
-        cont = False
+        print(
+            f"Meadowrun agent: Unexpected worker exit, restarting worker. In "
+            f"task #{task.task_id}, attempt #{task.attempt}, {traceback.format_exc()}"
+        )
+
+        await restart_worker(worker_server, worker_monitor)
+
+    print(
+        f"Meadowrun agent: Completed task #{task.task_id}, attempt #{task.attempt}, "
+        f"state {ProcessState.ProcessStateEnum.Name(process_state.state)}, max "
+        f"memory {process_state.max_memory_used_gb}GB "
+    )
 
     await _complete_task(result_queue, task, process_state)
-    return cont
+    return True
 
 
-async def worker_function_async(
+async def worker_function(
     request_queue: Queue,
     result_queue: Queue,
     public_address: str,
     log_file_name: str,
-    worker_server: AgentTaskWorkerServer,
-    get_worker_stats: Callable[[], Awaitable[Stats]],
+    worker_server: TaskWorkerServer,
+    worker_monitor: WorkerMonitor,
 ) -> None:
     pid = os.getpid()
     log_file_name = f"{public_address}:{log_file_name}"
 
     while await _worker_iteration(
-        request_queue, result_queue, log_file_name, pid, worker_server
+        request_queue, result_queue, log_file_name, pid, worker_server, worker_monitor
     ):
         pass
 
@@ -272,7 +279,7 @@ async def get_results_unordered(
     location: str,
     stop_receiving: asyncio.Event,
     all_workers_exited: asyncio.Event,
-) -> AsyncIterable[List[Tuple[int, int, ProcessState]]]:
+) -> AsyncIterable[Tuple[List[TaskProcessState], List[WorkerProcessState]]]:
 
     # TODO currently, we get back messages saying that a task is running on a particular
     # worker. We don't really do anything with these messages, but eventually we should
@@ -328,7 +335,7 @@ async def get_results_unordered(
                     num_tasks_running -= 1
                     num_tasks_completed += 1
                     results.append(
-                        (
+                        TaskProcessState(
                             task_result.task_id,
                             task_result.attempt,
                             task_result.process_state,
@@ -341,4 +348,4 @@ async def get_results_unordered(
                     message.pop_receipt,
                 )
 
-            yield results
+            yield results, []
