@@ -30,7 +30,7 @@ from meadowrun.aws_integration.management_lambdas.ec2_alloc_stub import (
     _MEADOWRUN_GENERATED_DOCKER_REPO,
 )
 from meadowrun.azure_integration.acr import get_acr_helper
-from meadowrun.conda import get_cached_or_create_conda_environment
+from meadowrun.deployment.conda import get_cached_or_create_conda_environment
 from meadowrun.credentials import RawCredentials, SshKey
 from meadowrun.docker_controller import (
     _does_digest_exist_locally,
@@ -47,14 +47,15 @@ from meadowrun.meadowrun_pb2 import (
     ServerAvailableContainer,
     ServerAvailableInterpreter,
 )
-from meadowrun.pip_integration import get_cached_or_create_pip_environment
-from meadowrun.poetry_integration import get_cached_or_create_poetry_environment
+from meadowrun.deployment.pip import get_cached_or_create_pip_environment
+from meadowrun.deployment.poetry import (
+    get_cached_or_create_poetry_environment,
+)
 from meadowrun.azure_integration.blob_storage import AzureBlobStorage
 from meadowrun.aws_integration.s3 import S3ObjectStorage
 from meadowrun.run_job_core import (
     CloudProviderType,
     ContainerRegistryHelper,
-    LocalObjectStorage,
 )
 from meadowrun.s3_grid_job import (
     try_get_storage_file,
@@ -62,6 +63,10 @@ from meadowrun.s3_grid_job import (
 )
 from meadowrun.func_worker_storage_helper import FuncWorkerClientObjectStorage
 import meadowrun.func_worker_storage_helper
+from meadowrun.deployment.prerequisites import (
+    EnvironmentSpecPrerequisites,
+    GOOGLE_AUTH_PACKAGE,
+)
 
 _GIT_REPO_URL_SUFFIXES_TO_REMOVE = [".git", "/"]
 
@@ -264,12 +269,11 @@ async def _get_git_code_paths(
 _ALL_OBJECT_STORAGES = {
     # see the note on get_url_scheme
     s.get_url_scheme(): s  # type: ignore
-    for s in [
+    for s in (
         AzureBlobStorage,
         FuncWorkerClientObjectStorage,
-        LocalObjectStorage,
         S3ObjectStorage,
-    ]
+    )
 }
 
 
@@ -308,7 +312,7 @@ async def compile_environment_spec_to_container(
     TODO we should also consider creating conda environments locally rather than always
     making a container for them, that might be more efficient.
     """
-    path_to_spec, spec_hash, has_git_dependency = _get_path_and_hash(
+    path_to_spec, spec_hash, prerequisites = _get_path_and_hash(
         environment_spec, interpreter_spec_path
     )
 
@@ -361,16 +365,15 @@ async def compile_environment_spec_to_container(
     apt_packages = [
         p for p in environment_spec.additional_software.keys() if p != "cuda"
     ]
-    if has_git_dependency:
+    if prerequisites & EnvironmentSpecPrerequisites.GIT:
         apt_packages.append("git")
     if apt_packages:
         build_args["APT_PACKAGES"] = " ".join(apt_packages)
+    if prerequisites & EnvironmentSpecPrerequisites.GOOGLE_AUTH:
+        build_args["ENVIRONMENT_PREREQUISITES"] = GOOGLE_AUTH_PACKAGE
 
     if environment_spec.environment_type == EnvironmentType.CONDA:
-        if apt_packages:
-            docker_file_name = "CondaAptDockerfile"
-        else:
-            docker_file_name = "CondaDockerfile"
+        docker_file_name = "CondaDockerfile"
         spec_filename = os.path.basename(path_to_spec)
         build_args["ENV_FILE"] = spec_filename
         files_to_copy.append((path_to_spec, spec_filename))
@@ -393,18 +396,12 @@ async def compile_environment_spec_to_container(
             ] = f"python:{environment_spec.python_version}-slim-bullseye"
 
         if environment_spec.environment_type == EnvironmentType.PIP:
-            if apt_packages:
-                docker_file_name = "PipAptDockerfile"
-            else:
-                docker_file_name = "PipDockerfile"
+            docker_file_name = "PipDockerfile"
             spec_filename = os.path.basename(path_to_spec)
             files_to_copy.append((path_to_spec, spec_filename))
             build_args["ENV_FILE"] = spec_filename
         elif environment_spec.environment_type == EnvironmentType.POETRY:
-            if apt_packages:
-                docker_file_name = "PoetryAptDockerfile"
-            else:
-                docker_file_name = "PoetryDockerfile"
+            docker_file_name = "PoetryDockerfile"
             files_to_copy.append(
                 (os.path.join(path_to_spec, "pyproject.toml"), "pyproject.toml")
             )
@@ -448,7 +445,7 @@ async def compile_environment_spec_locally(
     """
     Turns e.g. a conda_environment.yml file into a locally available python interpreter
     """
-    path_to_spec, spec_hash, has_git_dependency = _get_path_and_hash(
+    path_to_spec, spec_hash, prerequisites = _get_path_and_hash(
         environment_spec, interpreter_spec_path
     )
     # we don't have to worry about has_git_dependency as we assume we always have git
@@ -467,6 +464,7 @@ async def compile_environment_spec_locally(
             [
                 str,
                 str,
+                EnvironmentSpecPrerequisites,
                 str,
                 Callable[[str, str], Awaitable[bool]],
                 Callable[[str, str], Awaitable[None]],
@@ -486,6 +484,7 @@ async def compile_environment_spec_locally(
         interpreter_path=await get_cached_or_create(
             spec_hash,
             path_to_spec,
+            prerequisites,
             os.path.join(built_interpreters_folder, spec_hash),
             # TODO this isn't the right way to do this--these try_get_storage_file and
             # write_storage file functions should be getting passed in from higher up.
@@ -525,7 +524,7 @@ def _format_additional_software(additional_software: Mapping[str, str]) -> str:
 def _get_path_and_hash(
     environment_spec: Union[EnvironmentSpecInCode, EnvironmentSpec],
     interpreter_spec_path: str,
-) -> Tuple[str, str, bool]:
+) -> Tuple[str, str, EnvironmentSpecPrerequisites]:
     """
     Returns path_to_spec, spec_hash, has_git_dependency. spec_hash is a hash that
     uniquely identifies the environment. path_to_spec is used slightly differently for
@@ -554,7 +553,7 @@ def _get_path_and_hash(
                     environment_spec.additional_software
                 ).encode("utf-8")
             ),
-            _has_git_dependency(
+            _get_environment_spec_prerequisites(
                 spec_contents_bytes.decode("utf-8"), environment_spec.environment_type
             ),
         )
@@ -606,7 +605,9 @@ def _get_path_and_hash(
         return (
             path_to_spec,
             spec_hash,
-            _has_git_dependency(spec_contents_str, environment_spec.environment_type),
+            _get_environment_spec_prerequisites(
+                spec_contents_str, environment_spec.environment_type
+            ),
         )
     else:
         raise ValueError(
@@ -614,9 +615,11 @@ def _get_path_and_hash(
         )
 
 
-def _has_git_dependency(
+def _get_environment_spec_prerequisites(
     spec_contents: str, environment_type: EnvironmentType.ValueType
-) -> bool:
+) -> EnvironmentSpecPrerequisites:
+    result = EnvironmentSpecPrerequisites.NONE
+
     if (
         environment_type == EnvironmentType.PIP
         or environment_type == EnvironmentType.CONDA
@@ -624,10 +627,20 @@ def _has_git_dependency(
         # example line resulting from pip freeze:
         # package @ git+https://github.com/name/repo.git@sha
         # Conda can have git-based dependencies via pip
-        return "git+" in spec_contents
+        if "git+" in spec_contents:
+            result |= EnvironmentSpecPrerequisites.GIT
+
+        if "-python.pkg.dev" in spec_contents:
+            # TODO this could be more sophisticated, e.g. if they've already supplied a
+            # username/password, if they're not running on GCP, then we shouldn't set
+            # this flag
+            result |= EnvironmentSpecPrerequisites.GOOGLE_AUTH
     elif environment_type == EnvironmentType.POETRY:
-        return any(
-            line.startswith('type = "git"') for line in spec_contents.splitlines()
-        )
-    else:
-        return False
+        for line in spec_contents.splitlines():
+            if line.startswith('type = "git"'):
+                result |= EnvironmentSpecPrerequisites.GIT
+            elif line.startswith("url = ") and "-python.pkg.dev" in line:
+                result |= EnvironmentSpecPrerequisites.GOOGLE_AUTH
+            # TODO break early if we've found everything?
+
+    return result
