@@ -9,15 +9,9 @@ import collections
 import dataclasses
 import enum
 import itertools
-import os
-import os.path
 import pickle
-import shutil
-import sys
 import time
 import traceback
-import urllib.parse
-import zipfile
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -40,9 +34,9 @@ from typing import (
 
 import asyncssh
 import cloudpickle
-import filelock
 from typing_extensions import Literal
 
+
 import meadowrun.ssh as ssh
 from meadowrun.config import MEMORY_GB
 from meadowrun.instance_allocation import allocate_jobs_to_instances, InstanceRegistrar
@@ -52,7 +46,8 @@ from meadowrun.shared import unpickle_exception
 
 if TYPE_CHECKING:
     from meadowrun.credentials import UsernamePassword
-    from meadowrun.run_job_local import AgentTaskWorkerServer, Stats
+    from meadowrun.object_storage import ObjectStorage
+    from meadowrun.run_job_local import TaskWorkerServer, WorkerMonitor
     from types import TracebackType
 
 
@@ -138,132 +133,6 @@ class Resources:
             )
 
 
-class ObjectStorage(abc.ABC):
-    """An ObjectStorage is a place where you can upload files and download them"""
-
-    async def __aenter__(self) -> ObjectStorage:
-        return self
-
-    async def __aexit__(
-        self,
-        exc_typ: Type[BaseException],
-        exc_val: BaseException,
-        exc_tb: TracebackType,
-    ) -> None:
-        pass
-
-    @classmethod
-    @abc.abstractmethod
-    def get_url_scheme(cls) -> str:
-        """
-        Right now we're using the URL scheme to effectively serialize the ObjectStorage
-        object to the job. This works as long as we don't need any additional parameters
-        (region name, username/password), but we may need to make this more flexible in
-        the future.
-        """
-        pass
-
-    @abc.abstractmethod
-    async def upload_from_file_url(self, file_url: str) -> str:
-        """
-        file_url will be a file:// url to a file on the local machine. This function
-        should upload that file to the object storage, delete the local file, and return
-        the URL of the remote file.
-        """
-        pass
-
-    @abc.abstractmethod
-    async def download_and_unzip(
-        self, remote_url: str, local_copies_folder: str
-    ) -> str:
-        """
-        remote_url will be the URL of a file in the object storage system as generated
-        by upload_from_file_url. This function should download the file and extract it
-        to local_copies_folder if it has not already been extracted.
-        """
-        pass
-
-
-class LocalObjectStorage(ObjectStorage):
-    """
-    This is a "pretend" version of ObjectStorage where we assume that we have the same
-    file system available on both the client and the server. Mostly for testing.
-    """
-
-    @classmethod
-    def get_url_scheme(cls) -> str:
-        return "file"
-
-    async def upload_from_file_url(self, file_url: str) -> str:
-        # TODO maybe assert that this starts with file://
-        return file_url
-
-    async def download_and_unzip(
-        self, remote_url: str, local_copies_folder: str
-    ) -> str:
-        decoded_url = urllib.parse.urlparse(remote_url)
-        extracted_folder = os.path.join(
-            local_copies_folder, os.path.splitext(os.path.basename(decoded_url.path))[0]
-        )
-        with filelock.FileLock(f"{extracted_folder}.lock", timeout=120):
-            if not os.path.exists(extracted_folder):
-                with zipfile.ZipFile(decoded_url.path) as zip_file:
-                    zip_file.extractall(extracted_folder)
-
-        return extracted_folder
-
-
-class S3CompatibleObjectStorage(ObjectStorage, abc.ABC):
-    """
-    Think of this class as what ObjectStorage "should be" if it weren't for
-    LocalObjectStorage. Most implementations of ObjectStorage should implement this
-    class.
-    """
-
-    @abc.abstractmethod
-    async def _upload(self, file_path: str) -> Tuple[str, str]:
-        pass
-
-    @abc.abstractmethod
-    async def _download(
-        self, bucket_name: str, object_name: str, file_name: str
-    ) -> None:
-        pass
-
-    async def upload_from_file_url(self, file_url: str) -> str:
-        decoded_url = urllib.parse.urlparse(file_url)
-        if decoded_url.scheme != "file":
-            raise ValueError(f"Expected file URI: {file_url}")
-        if sys.platform == "win32" and decoded_url.path.startswith("/"):
-            # on Windows, file:///C:\foo turns into file_url.path = /C:\foo so we need
-            # to remove the forward slash at the beginning
-            file_path = decoded_url.path[1:]
-        else:
-            file_path = decoded_url.path
-        bucket_name, object_name = await self._upload(file_path)
-        shutil.rmtree(os.path.dirname(file_path), ignore_errors=True)
-        return urllib.parse.urlunparse(
-            (self.get_url_scheme(), bucket_name, object_name, "", "", "")
-        )
-
-    async def download_and_unzip(
-        self, remote_url: str, local_copies_folder: str
-    ) -> str:
-        decoded_url = urllib.parse.urlparse(remote_url)
-        bucket_name = decoded_url.netloc
-        object_name = decoded_url.path.lstrip("/")
-        extracted_folder = os.path.join(local_copies_folder, object_name)
-
-        with filelock.FileLock(f"{extracted_folder}.lock", timeout=120):
-            if not os.path.exists(extracted_folder):
-                zip_file_path = extracted_folder + ".zip"
-                await self._download(bucket_name, object_name, zip_file_path)
-                with zipfile.ZipFile(zip_file_path) as zip_file:
-                    zip_file.extractall(extracted_folder)
-
-        return extracted_folder
-
-
 class WaitOption(enum.Enum):
     WAIT_AND_TAIL_STDOUT = 1
     WAIT_SILENTLY = 2
@@ -597,7 +466,10 @@ class MeadowrunException(Exception):
     def __init__(
         self, process_state: ProcessState, address: Optional[str] = None
     ) -> None:
-        if process_state.state == ProcessState.ProcessStateEnum.NON_ZERO_RETURN_CODE:
+        if process_state.state in (
+            ProcessState.ProcessStateEnum.NON_ZERO_RETURN_CODE,
+            ProcessState.ProcessStateEnum.UNEXPECTED_WORKER_EXIT,
+        ):
             return_code = f": {process_state.return_code}"
         else:
             return_code = ""
@@ -615,10 +487,14 @@ class MeadowrunException(Exception):
         elif address is not None:
             message.append(f"On host: {address}")
 
-        if process_state.state in (
-            ProcessState.ProcessStateEnum.RUN_REQUEST_FAILED,
-            ProcessState.ProcessStateEnum.PYTHON_EXCEPTION,
-            ProcessState.ProcessStateEnum.ERROR_GETTING_STATE,
+        if (
+            process_state.state
+            in (
+                ProcessState.ProcessStateEnum.RUN_REQUEST_FAILED,
+                ProcessState.ProcessStateEnum.PYTHON_EXCEPTION,
+                ProcessState.ProcessStateEnum.ERROR_GETTING_STATE,
+            )
+            and process_state.pickled_result
         ):
             remote_exception = unpickle_exception(process_state.pickled_result)
             message.append(remote_exception[2])
@@ -660,36 +536,52 @@ class TaskResult(Generic[_T]):
     log_file_name: str = ""
 
     @staticmethod
-    def from_process_state(
-        task_id: int, attempt: int, result: ProcessState
-    ) -> TaskResult:
-        if result.state == ProcessState.ProcessStateEnum.SUCCEEDED:
-            # TODO try/catch on pickle.loads?
+    def from_process_state(task: TaskProcessState) -> TaskResult:
+        if task.result.state == ProcessState.ProcessStateEnum.SUCCEEDED:
+
+            def _safe_pickle_loads(pickled: bytes) -> Any:
+                try:
+                    return pickle.loads(pickled)
+                except:  # noqa: E722
+                    return (
+                        "Task was successfully executed, but pickle.loads of result "
+                        f"failed. Error: {traceback.format_exc()}"
+                    )
+
             return TaskResult(
-                task_id,
+                task.task_id,
                 is_success=True,
-                state=ProcessState.ProcessStateEnum.Name(result.state),
-                result=pickle.loads(result.pickled_result),
-                attempt=attempt,
-                log_file_name=result.log_file_name,
+                state=ProcessState.ProcessStateEnum.Name(task.result.state),
+                result=_safe_pickle_loads(task.result.pickled_result),
+                attempt=task.attempt,
+                log_file_name=task.result.log_file_name,
             )
-        elif result.state in _EXCEPTION_STATES:
-            exception = unpickle_exception(result.pickled_result)
+        elif task.result.state in _EXCEPTION_STATES:
+            exception = unpickle_exception(task.result.pickled_result)
+            if exception is None and task.result.return_code != 0:
+                return_code_message = f"Non-zero return code: {task.result.return_code}"
+                exception = ("", return_code_message, return_code_message + "\n")
             return TaskResult(
-                task_id,
+                task.task_id,
                 is_success=False,
-                state=ProcessState.ProcessStateEnum.Name(result.state),
+                state=ProcessState.ProcessStateEnum.Name(task.result.state),
                 exception=exception,
-                attempt=attempt,
-                log_file_name=result.log_file_name,
+                attempt=task.attempt,
+                log_file_name=task.result.log_file_name,
             )
         else:
+            if task.result.return_code != 0:
+                return_code_message = f"Non-zero return code: {task.result.return_code}"
+                exception = ("", return_code_message, return_code_message + "\n")
+            else:
+                exception = None
             return TaskResult(
-                task_id,
+                task.task_id,
                 is_success=False,
-                state=ProcessState.ProcessStateEnum.Name(result.state),
-                attempt=attempt,
-                log_file_name=result.log_file_name,
+                state=ProcessState.ProcessStateEnum.Name(task.result.state),
+                exception=exception,
+                attempt=task.attempt,
+                log_file_name=task.result.log_file_name,
             )
 
     def result_or_raise(self) -> _T:
@@ -825,6 +717,19 @@ class AllocVM(Host, abc.ABC):
         pass
 
 
+@dataclasses.dataclass(frozen=True)
+class TaskProcessState:
+    task_id: int
+    attempt: int
+    result: ProcessState
+
+
+@dataclasses.dataclass(frozen=True)
+class WorkerProcessState:
+    worker_index: str
+    result: ProcessState
+
+
 class GridJobCloudInterface(abc.ABC, Generic[_T, _U]):
     """
     See also GridJobDriver. The GridJobDriver is a concrete class that handles the
@@ -875,7 +780,7 @@ class GridJobCloudInterface(abc.ABC, Generic[_T, _U]):
     async def get_worker_function(
         self, queue_index: int
     ) -> Callable[
-        [str, str, AgentTaskWorkerServer, Callable[[], Awaitable[Stats]]],
+        [str, str, TaskWorkerServer, WorkerMonitor],
         Coroutine[Any, Any, None],
     ]:
         """
@@ -888,7 +793,7 @@ class GridJobCloudInterface(abc.ABC, Generic[_T, _U]):
     @abc.abstractmethod
     async def receive_task_results(
         self, *, stop_receiving: asyncio.Event, workers_done: asyncio.Event
-    ) -> AsyncIterable[List[Tuple[int, int, ProcessState]]]:
+    ) -> AsyncIterable[Tuple[List[TaskProcessState], List[WorkerProcessState]]]:
         ...
 
     @abc.abstractmethod
@@ -1070,10 +975,6 @@ class GridJobDriver:
                     job, wait_for_result, deallocator
                 )
 
-            workers_needed_changed_wait_task = asyncio.create_task(
-                self._num_workers_needed_changed.wait()
-            )
-
             async with self._cloud_interface.create_instance_registrar() as instance_registrar:  # noqa: E501
                 while True:
                     for worker_queue in self._worker_queues:
@@ -1193,6 +1094,12 @@ class GridJobDriver:
                             new_worker_tasks.append(worker_task)
                     worker_tasks = new_worker_tasks
 
+                    if workers_needed_changed_wait_task.done():
+                        self._num_workers_needed_changed.clear()
+                        workers_needed_changed_wait_task = asyncio.create_task(
+                            self._num_workers_needed_changed.wait()
+                        )
+
                     # this means all workers are either done or shutting down
                     if all(
                         worker_queue.num_workers_launched
@@ -1274,52 +1181,61 @@ class GridJobDriver:
             f"Waiting for task results. Requested: {len(args)}, "
             f"Done: {num_tasks_done}"
         )
-        async for batch in await self._cloud_interface.receive_task_results(  # noqa: E501
+        async for task_batch, worker_batch in await self._cloud_interface.receive_task_results(  # noqa: E501
             stop_receiving=stop_receiving, workers_done=self._no_workers_available
         ):
-            for task_id, attempt, result in batch:
-                task_result = TaskResult.from_process_state(task_id, attempt, result)
+            # TODO right now we ignore worker_batch because we get worker failures
+            # through the SSH connection. At some point, we may want to process worker
+            # failures here.
+
+            for task in task_batch:
+                task_result = TaskResult.from_process_state(task)
                 if task_result.is_success:
                     num_tasks_done += 1
-                    arg_to_queue_index[task_id] = -1
+                    arg_to_queue_index[task.task_id] = -1
                     yield task_result
-                elif attempt < max_num_task_attempts:
-                    prev_queue_index = arg_to_queue_index[task_id]
+                elif task.attempt < max_num_task_attempts:
+                    prev_queue_index = arg_to_queue_index[task.task_id]
                     prev_memory_requirement = _memory_gb_for_queue_index(
                         prev_queue_index, self._resources_required_per_task
                     )
                     if (
                         retry_with_more_memory
-                        and result.max_memory_used_gb >= 0.95 * prev_memory_requirement
+                        and task.result.max_memory_used_gb
+                        >= 0.95 * prev_memory_requirement
                     ):
                         print(
-                            f"Task {task_id} failed at attempt {attempt}, retrying with"
-                            f" more memory (task used {result.max_memory_used_gb:.2f}/"
+                            f"Task {task.task_id} failed at attempt {task.attempt}, "
+                            "retrying with more memory (task used "
+                            f"{task.result.max_memory_used_gb:.2f}/"
                             f"{prev_memory_requirement}GB requested)"
                         )
 
-                        new_queue_index = arg_to_queue_index[task_id] + 1
-                        arg_to_queue_index[task_id] = new_queue_index
+                        new_queue_index = arg_to_queue_index[task.task_id] + 1
+                        arg_to_queue_index[task.task_id] = new_queue_index
                         if len(self._worker_queues) < new_queue_index + 1:
                             # TODO any new queue gets 1 worker by default. We should
                             # increase this later depending on how the situation changes
                             self._cloud_interface.create_queue()
                             self._worker_queues.append(WorkerQueue(new_queue_index, 1))
                         await self._cloud_interface.retry_task(
-                            task_id, attempt, new_queue_index
+                            task.task_id, task.attempt, new_queue_index
                         )
                     else:
-                        print(f"Task {task_id} failed at attempt {attempt}, retrying")
+                        print(
+                            f"Task {task.task_id} failed at attempt {task.attempt}, "
+                            "retrying"
+                        )
                         await self._cloud_interface.retry_task(
-                            task_id, attempt, prev_queue_index
+                            task.task_id, task.attempt, prev_queue_index
                         )
                 else:
                     print(
-                        f"Task {task_id} failed at attempt {attempt}, "
-                        f"max attempts is {max_num_task_attempts}, not retrying."
+                        f"Task {task.task_id} failed at attempt {task.attempt}, max "
+                        f"attempts is {max_num_task_attempts}, not retrying."
                     )
                     num_tasks_done += 1
-                    arg_to_queue_index[task_id] = -1
+                    arg_to_queue_index[task.task_id] = -1
                     yield task_result
 
             if num_tasks_done >= len(args):
