@@ -1,9 +1,9 @@
 from __future__ import annotations
 
+import abc
 import asyncio
 import asyncio.subprocess
 import dataclasses
-import functools
 import itertools
 import os
 import os.path
@@ -13,10 +13,11 @@ import shutil
 import struct
 import sys
 import traceback
+from decimal import InvalidOperation
 from typing import (
+    TYPE_CHECKING,
     Any,
     AsyncIterable,
-    Awaitable,
     Callable,
     Coroutine,
     Dict,
@@ -24,7 +25,6 @@ from typing import (
     List,
     Optional,
     Sequence,
-    TYPE_CHECKING,
     Tuple,
     TypeVar,
     Union,
@@ -41,8 +41,10 @@ if TYPE_CHECKING:
     JobSpecType = Literal["py_command", "py_function", "py_agent"]
 
     from meadowrun._vendor import aiodocker
-    from meadowrun._vendor.aiodocker import containers as aiodocker_containers
+
+    from meadowrun._vendor.aiodocker.containers import DockerContainer
     from meadowrun.instance_selection import ResourcesInternal
+    from meadowrun.object_storage import ObjectStorage
 
 from meadowrun.config import (
     MEADOWRUN_AGENT_PID,
@@ -72,21 +74,19 @@ from meadowrun.meadowrun_pb2 import (
     Credentials,
     Job,
     ProcessState,
-    PyFunctionJob,
     PyAgentJob,
+    PyFunctionJob,
     StringPair,
 )
 from meadowrun.run_job_core import (
     CloudProviderType,
     Host,
     JobCompletion,
-    LocalObjectStorage,
     MeadowrunException,
-    ObjectStorage,
     TaskResult,
     WaitOption,
 )
-from meadowrun.shared import pickle_exception
+from meadowrun.shared import cancel_task, pickle_exception
 
 ProcessStateEnum = ProcessState.ProcessStateEnum
 
@@ -129,7 +129,7 @@ class _JobSpecTransformed:
     environment_variables: Dict[str, str] = dataclasses.field(
         default_factory=lambda: {}
     )
-    server: Optional[AgentTaskWorkerServer] = None
+    server: Optional[TaskWorkerServer] = None
 
 
 def _io_file_container_binds(
@@ -347,7 +347,9 @@ async def _prepare_py_agent(
         for io_file in io_files:
             open(os.path.join(io_folder, io_file), "w", encoding="utf-8").close()
 
-    server = await agent_start_serving("0.0.0.0" if is_container else "127.0.0.1")
+    server = await TaskWorkerServer.start_serving(
+        "0.0.0.0" if is_container else "127.0.0.1"
+    )
     command_line = [
         "python",
         worker_path,
@@ -376,9 +378,88 @@ async def _prepare_py_agent(
     )
 
 
-@dataclasses.dataclass
-class StatsAccumulator:
-    max_memory_used_gb: float = 0
+class TaskWorkerServer:
+    """This class opens a port in the agent process for the task worker to connect to.
+    It allows waiting for connections from task workers, and sending and receiving
+    messages.
+    """
+
+    def __init__(self) -> None:
+        self.server: Optional[asyncio.AbstractServer] = None
+        self.reader: Optional[asyncio.StreamReader] = None
+        self.writer: Optional[asyncio.StreamWriter] = None
+        self.have_connection: asyncio.Event = asyncio.Event()
+        self.port: Optional[int] = None
+
+    @staticmethod
+    async def start_serving(
+        host: str,
+    ) -> TaskWorkerServer:
+        """Create a TaskWorkerServer, open a free port, and return the server."""
+        server = TaskWorkerServer()
+        await server._start_serving(host)
+        return server
+
+    async def _start_serving(
+        self,
+        host: str,
+    ) -> None:
+        server = await asyncio.start_server(self._handle_connection, host, 0)
+        (socket,) = server.sockets
+        self.port = socket.getsockname()[1]
+
+    async def _handle_connection(
+        self, reader: asyncio.StreamReader, writer: asyncio.StreamWriter
+    ) -> None:
+        self.reader = reader
+        self.writer = writer
+        self.have_connection.set()
+
+    async def wait_for_task_worker_connection(
+        self, timeout: float = 30
+    ) -> Tuple[asyncio.StreamReader, asyncio.StreamWriter]:
+        """Wait until a task worker process connects, with an optional timeout."""
+        await asyncio.wait_for(self.have_connection.wait(), timeout=timeout)
+        assert self.reader is not None
+        assert self.writer is not None
+        return self.reader, self.writer
+
+    async def send_message(self, bs: bytes) -> None:
+        """Send the given bytes as a message to the task worker. If no task worker is
+        connected yet, waits for a connection."""
+        if not self.have_connection.is_set():
+            await self.wait_for_task_worker_connection()
+        assert self.writer is not None
+        msg_len = struct.pack(">i", len(bs))
+        self.writer.write(msg_len)
+        self.writer.write(bs)
+        await self.writer.drain()
+
+    async def receive_message(self) -> Any:
+        """Receives a message from the task worker. If no task worker is
+        connected yet, waits for a connection. The message is unpickled."""
+        if not self.have_connection.is_set():
+            await self.wait_for_task_worker_connection()
+        assert self.reader is not None
+        (result_len,) = struct.unpack(">i", await self.reader.read(4))
+        result_bs = bytearray()
+        while len(result_bs) < result_len:
+            result_bs.extend(await self.reader.read(result_len - len(result_bs)))
+        return pickle.loads(result_bs)
+
+    async def close_task_worker_connection(self) -> None:
+        """Cleanly close the connection to the task worker."""
+        if self.writer is not None:
+            self.writer.close()
+            await self.writer.wait_closed()
+            self.have_connection.clear()
+
+    async def close(self) -> None:
+        """Cleanly close the connection to the task worker, and stop serving."""
+        await self.close_task_worker_connection()
+        if self.server:
+            self.server.close()
+            await self.server.wait_closed()
 
 
 @dataclasses.dataclass(frozen=True)
@@ -387,6 +468,273 @@ class Stats:
     memory_in_use_gb: float
 
 
+@dataclasses.dataclass
+class StatsAccumulator:
+    max_memory_used_gb: float = 0
+
+    def accumulate(self, stats: Stats) -> None:
+        if stats.memory_in_use_gb > self.max_memory_used_gb:
+            self.max_memory_used_gb = stats.memory_in_use_gb
+
+    def snapshot(self) -> Stats:
+        return Stats(memory_in_use_gb=self.max_memory_used_gb)
+
+
+class WorkerMonitor(abc.ABC):
+    """Monitors a task worker."""
+
+    def __init__(self) -> None:
+        super().__init__()
+        self._stats_accumulator = StatsAccumulator()
+        self._stats_task: Optional[asyncio.Task[None]] = None
+
+    @abc.abstractmethod
+    async def wait_until_exited(self) -> int:
+        ...
+
+    @abc.abstractmethod
+    async def try_get_return_code(self) -> Optional[int]:
+        """
+        Returns the exit code of the process if it has already exited, otherwise returns
+        None
+        """
+        ...
+
+    @abc.abstractmethod
+    async def restart(self) -> None:
+        """Restart the task worker."""
+        ...
+
+    @abc.abstractmethod
+    async def get_stats(self) -> Stats:
+        """Get some stats (memory, cpu...) about the task worker."""
+        ...
+
+    async def _start_stats(self) -> None:
+        """Subclasses should start this as a task to accumulate statistics,
+        and cancel the task when done."""
+        self._stats_accumulator = StatsAccumulator()
+        while True:
+            try:
+                stats = await self.get_stats()
+                self._stats_accumulator.accumulate(stats)
+            except asyncio.CancelledError:
+                raise
+            except Exception:
+                print("Error updating stats:\n" + traceback.format_exc())
+
+            await asyncio.sleep(1)
+
+    def start_stats(self) -> None:
+        self._stats_task = asyncio.create_task(self._start_stats())
+
+    async def stop_stats(self) -> StatsAccumulator:
+        if self._stats_task is not None:
+            await cancel_task(self._stats_task)
+        return self._stats_accumulator
+
+
+class WorkerProcessMonitor(WorkerMonitor):
+    """Monitors a task worker that runs as a process."""
+
+    def __init__(
+        self,
+        command_line: Iterable[str],
+        working_directory: Optional[str],
+        env_vars: Dict[str, str],
+    ):
+        super().__init__()
+        self.command_line = tuple(command_line)
+        self.working_directory = working_directory
+        self.env_vars = env_vars
+        self._process: Optional[asyncio.subprocess.Process] = None
+        self._tail_task: Optional[asyncio.Task[None]] = None
+
+    @property
+    def process(self) -> asyncio.subprocess.Process:
+        if self._process is None:
+            raise InvalidOperation(
+                "Worker process not started. Call start_and_tail first."
+            )
+        return self._process
+
+    @property
+    def pid(self) -> int:
+        return self.process.pid
+
+    async def start_and_tail(self) -> None:
+        self._process = await asyncio.subprocess.create_subprocess_exec(
+            *self.command_line,
+            stdout=asyncio.subprocess.PIPE,
+            stderr=asyncio.subprocess.STDOUT,
+            cwd=self.working_directory,
+            env=self.env_vars,
+        )
+        self._tail_task = asyncio.create_task(self._tail_log())
+
+    async def stop(self) -> None:
+        """Safely stop the process."""
+        await self.stop_stats()
+        try:
+            if self._process is not None:
+                self._process.terminate()
+                self._process.kill()
+        except ProcessLookupError:
+            # if it's already gone, we're done here
+            pass
+
+    async def wait_until_exited(self) -> int:
+        returncode = await self.process.wait()
+        if self._tail_task is not None:
+            await self._tail_task
+        self._tail_task = None
+        return returncode
+
+    async def try_get_return_code(self) -> Optional[int]:
+        return self.process.returncode
+
+    async def cleanup(self) -> None:
+        if self._process is not None:
+            await self.stop()
+            await self.wait_until_exited()
+        if self._tail_task is not None:
+            self._tail_task.cancel()
+            try:
+                await self._tail_task
+            except asyncio.CancelledError:
+                pass
+        self._tail_task = None
+
+    async def _tail_log(self) -> None:
+        process = self.process
+        assert process.stdout is not None
+        async for line in process.stdout:
+            sys.stdout.buffer.write(line)
+
+    async def restart(self) -> None:
+        await self.stop()
+        await self.wait_until_exited()
+        await self.start_and_tail()
+
+    async def get_stats(self) -> Stats:
+        # TODO cache the psutil process
+        process = psutil.Process(self.process.pid)
+        # RSS is in bytes: https://man7.org/linux/man-pages/man1/ps.1.html
+        return Stats(process.memory_info().rss / (1024**3))
+
+
+class WorkerContainerMonitor(WorkerMonitor):
+    """Monitors a task worker that runs as a container."""
+
+    def __init__(
+        self,
+        client: Optional[aiodocker.Docker],
+        image: str,
+        cmd: Optional[List[str]],
+        environment_variables: Dict[str, str],
+        working_directory: Optional[str],
+        binds: List[Tuple[str, str]],
+        ports: Iterable[str],
+        extra_hosts: List[Tuple[str, str]],
+        uses_gpus: bool,
+    ):
+        self.docker_client = client
+        self.image = image
+        self.command_line = cmd
+        self.env_vars = environment_variables
+        self.working_directory = working_directory
+        self.binds = binds
+        self.ports = list(ports)
+        self.extra_hosts = extra_hosts
+        self.uses_gpus = uses_gpus
+        self._container: Optional[DockerContainer] = None
+        self._tail_task: Optional[asyncio.Task[None]] = None
+
+    @property
+    def container(self) -> DockerContainer:
+        if self._container is None:
+            raise InvalidOperation(
+                "Container not started yet. Call start_and_tail first."
+            )
+        return self._container
+
+    @property
+    def container_id(self) -> str:
+        return self.container.id
+
+    async def start_and_tail(self) -> None:
+        self._container, self.docker_client = await run_container(
+            self.docker_client,
+            self.image,
+            self.command_line,
+            self.env_vars,
+            self.working_directory,
+            self.binds,
+            self.ports,
+            self.extra_hosts,
+            self.uses_gpus,
+        )
+        self._tail_task = asyncio.create_task(self._tail_log())
+
+    async def stop(self) -> None:
+        if self._container is not None:
+            await self.container.stop()
+
+    async def wait_until_exited(self) -> int:
+        wait_result = await self.container.wait()
+        if self._tail_task is not None:
+            await self._tail_task
+        self.tail_task = None
+        # as per https://docs.docker.com/engine/api/v1.41/#operation/ContainerWait we
+        # can get the return code from the result
+        return wait_result["StatusCode"]
+
+    async def try_get_return_code(self) -> Optional[int]:
+        container_status = (await self.container.show()).get("State", {})
+        if container_status.get("Status", "") == "running":
+            # container_status will have ExitCode == 0 while the process is running
+            return None
+        else:
+            return container_status.get("ExitCode", None)
+
+    async def _tail_log(self) -> None:
+        # 1. Docker appears to have an objection to having a log driver that can
+        # produce plain text files (https://github.com/moby/moby/issues/17020) so we
+        # implement that in a hacky way here.
+        # TODO figure out overall strategy for logging, maybe eventually implement our
+        # own plain text/whatever log driver for docker.
+        async for line in self.container.log(stdout=True, stderr=True, follow=True):
+            print(f"Task worker: {line}", end="")
+
+    async def restart(self) -> None:
+        await self.stop()
+        await self.wait_until_exited()
+        await self.start_and_tail()
+
+    async def get_stats(self) -> Stats:
+        stats = await self.container.stats(stream=False)
+        if not stats:
+            return Stats(0)
+        # according to
+        # https://docs.docker.com/engine/api/v1.41/#tag/Container/operation/ContainerStats
+        # "memory used" in the docker CLI corresponds to this formula:
+        return Stats(
+            (
+                stats[0]["memory_stats"]["usage"]
+                - stats[0]["memory_stats"]["stats"]["cache"]
+            )
+            / (1024**3)
+        )
+
+
+async def restart_worker(
+    server: TaskWorkerServer, worker_monitor: WorkerMonitor
+) -> None:
+    await server.close_task_worker_connection()
+    await worker_monitor.restart()
+    await server.wait_for_task_worker_connection()
+
+
 async def _launch_non_container_job(
     job_spec_type: JobSpecType,
     job_spec_transformed: _JobSpecTransformed,
@@ -473,17 +821,14 @@ async def _launch_non_container_job(
         f"code paths={','.join(code_paths)}"
     )
 
-    process = await asyncio.subprocess.create_subprocess_exec(
-        *job_spec_transformed.command_line,
-        stdout=asyncio.subprocess.PIPE,
-        stderr=asyncio.subprocess.STDOUT,
-        cwd=working_directory,
-        env=env_vars,
+    worker = WorkerProcessMonitor(
+        job_spec_transformed.command_line, working_directory, env_vars
     )
+    await worker.start_and_tail()
 
     # (5) return the pid and continuation
-    return process.pid, _non_container_job_continuation(
-        process,
+    return worker.pid, _non_container_job_continuation(
+        worker,
         job_spec_type,
         job_spec_transformed,
         job,
@@ -492,24 +837,8 @@ async def _launch_non_container_job(
     )
 
 
-async def _stop_process(process: asyncio.subprocess.Process) -> None:
-    """Safe way to terminate and kill a process."""
-    try:
-        process.terminate()
-        process.kill()
-    except ProcessLookupError:
-        # if it's already gone, we're done here
-        pass
-
-
-async def _get_non_container_stats(pid: int) -> Stats:
-    process = psutil.Process(pid)
-    # RSS is in bytes: https://man7.org/linux/man-pages/man1/ps.1.html
-    return Stats(process.memory_info().rss / (1024**3))
-
-
 async def _non_container_job_continuation(
-    process: asyncio.subprocess.Process,
+    worker: WorkerProcessMonitor,
     job_spec_type: JobSpecType,
     job_spec_transformed: _JobSpecTransformed,
     job: Job,
@@ -523,30 +852,25 @@ async def _non_container_job_continuation(
     """
 
     try:
-
-        async def tail_log() -> None:
-            async for line in process.stdout:  # type: ignore
-                print(line)
-
-        tail_log_task = asyncio.create_task(tail_log())
-
         if job_spec_type == "py_agent":
-            await _run_agent(
-                job_spec_transformed,
-                job,
-                functools.partial(_get_non_container_stats, process.pid),
+            await _run_agent(job_spec_transformed, job, worker)
+            await worker.stop()
+            await worker.wait_until_exited()
+            return ProcessState(
+                state=ProcessStateEnum.SUCCEEDED,
+                pid=worker.pid or 0,
+                log_file_name=log_file_name,
+                return_code=0,
             )
-            await _stop_process(process)
 
-        returncode = await process.wait()
-        await tail_log_task
+        returncode = await worker.wait_until_exited()
         return _completed_job_state(
             job_spec_type,
             job.job_id,
             io_folder,
             log_file_name,
             returncode,
-            process.pid,
+            worker.pid,
             None,
         )
     except asyncio.CancelledError:
@@ -560,6 +884,13 @@ async def _non_container_job_continuation(
             state=ProcessStateEnum.ERROR_GETTING_STATE,
             pickled_result=pickle_exception(e, job.result_highest_pickle_protocol),
         )
+    finally:
+        # TODO this is not 100% bulletproof--there will be a tiny sliver of time between
+        # when we create the process and we enter this try/finally block
+        try:
+            await worker.stop()
+        except BaseException:
+            print("Exception trying to kill process: " + traceback.format_exc())
 
 
 async def _launch_container_job(
@@ -685,7 +1016,7 @@ async def _launch_container_job(
         f"ports={','.join(port for port in job.ports)}"
     )
 
-    container, docker_client = await run_container(
+    worker = WorkerContainerMonitor(
         docker_client,
         container_image_name,
         # json serializer needs a real list, not a protobuf fake list
@@ -697,45 +1028,27 @@ async def _launch_container_job(
         [(f"sidecar-container-{i}", ip) for i, ip in enumerate(sidecar_container_ips)],
         job.uses_gpu,
     )
-    return container.id, _container_job_continuation(
-        container,
-        docker_client,
+    await worker.start_and_tail()
+
+    return worker.container_id, _container_job_continuation(
+        worker,
         job_spec_type,
         job_spec_transformed,
         job,
-        # job.job_id,
         io_folder,
-        # job.result_highest_pickle_protocol,
         log_file_name,
         sidecar_containers,
     )
 
 
-async def _get_container_stats(
-    container: aiodocker_containers.DockerContainer,
-) -> Stats:
-    stats = await container.stats(stream=False)
-    if not stats:
-        return Stats(0)
-    # according to
-    # https://docs.docker.com/engine/api/v1.41/#tag/Container/operation/ContainerStats
-    # "memory used" in the docker CLI corresponds to this formula:
-    return Stats(
-        (stats[0]["memory_stats"]["usage"] - stats[0]["memory_stats"]["stats"]["cache"])
-        / (1024**3)
-    )
-
-
 async def _container_job_continuation(
-    container: aiodocker_containers.DockerContainer,
-    docker_client: aiodocker.Docker,
+    worker: WorkerContainerMonitor,
     job_spec_type: JobSpecType,
     job_spec_transformed: _JobSpecTransformed,
     job: Job,
     io_folder: str,
-    # result_highest_pickle_protocol: int,
     log_file_name: str,
-    sidecar_containers: List[aiodocker_containers.DockerContainer],
+    sidecar_containers: List[DockerContainer],
 ) -> ProcessState:
     """
     Writes the container's logs to log_file_name, waits for the container to finish, and
@@ -745,34 +1058,18 @@ async def _container_job_continuation(
     docker_client just needs to be closed when the container process has completed.
     """
     try:
-
-        async def tail_log() -> None:
-            # 1. Docker appears to have an objection to having a log driver that can
-            # produce plain text files (https://github.com/moby/moby/issues/17020) so we
-            # implement that in a hacky way here.
-            # TODO figure out overall strategy for
-            # logging, maybe eventually implement our own plain text/whatever log driver
-            #  for docker.
-            async for line in container.log(stdout=True, stderr=True, follow=True):
-                print(f"Task worker: {line}", end="")
-
-        tail_log_task = asyncio.create_task(tail_log())
-
         if job_spec_type == "py_agent":
-            await _run_agent(
-                job_spec_transformed,
-                job,
-                functools.partial(_get_container_stats, container),
+            await _run_agent(job_spec_transformed, job, worker)
+            await worker.stop()
+            await worker.wait_until_exited()
+            return ProcessState(
+                state=ProcessStateEnum.SUCCEEDED,
+                container_id=worker.container_id or "",
+                log_file_name=log_file_name,
+                return_code=0,
             )
-            await container.stop()
-
-        wait_result = await container.wait()
-        await tail_log_task
-
-        # as per https://docs.docker.com/engine/api/v1.41/#operation/ContainerWait we
-        # can get the return code from the result
-        return_code = wait_result["StatusCode"]
 
+        return_code = await worker.wait_until_exited()
         return _completed_job_state(
             job_spec_type,
             job.job_id,
@@ -780,7 +1077,7 @@ async def _container_job_continuation(
             log_file_name,
             return_code,
             None,
-            container.id,
+            worker.container_id,
         )
     except asyncio.CancelledError:
         raise
@@ -794,20 +1091,21 @@ async def _container_job_continuation(
             pickled_result=pickle_exception(e, job.result_highest_pickle_protocol),
         )
     finally:
+        # TODO this is not 100% bulletproof--there will be a tiny sliver of time between
+        # when we create the container and we enter this try/finally block
         try:
             await asyncio.gather(
-                remove_container(container),
+                remove_container(worker.container),
                 *(remove_container(c) for c in sidecar_containers),
             )
-        except Exception as e:
-            print(f"Warning, unable to remove container: {e}")
-        await docker_client.__aexit__(None, None, None)
+        except BaseException:
+            print("Warning, unable to remove container: " + traceback.format_exc())
+        assert worker.docker_client is not None
+        await worker.docker_client.__aexit__(None, None, None)
 
 
 async def _run_agent(
-    job_spec_transformed: _JobSpecTransformed,
-    job: Job,
-    get_worker_stats: Callable[[], Awaitable[Stats]],
+    job_spec_transformed: _JobSpecTransformed, job: Job, worker: WorkerMonitor
 ) -> None:
     # run the agent function. The agent function connects to another
     # process, the task worker, which runs the actual user function.
@@ -819,7 +1117,7 @@ async def _run_agent(
     await agent_func(
         *agent_func_args,
         worker_server=job_spec_transformed.server,
-        get_worker_stats=get_worker_stats,
+        worker_monitor=worker,
         **agent_func_kwargs,
     )
     await job_spec_transformed.server.close()
@@ -1059,72 +1357,6 @@ async def _get_credentials_for_job(
     )
 
 
-class AgentTaskWorkerServer:
-    def __init__(self) -> None:
-        self.server: Optional[asyncio.AbstractServer] = None
-        self.reader: Optional[asyncio.StreamReader] = None
-        self.writer: Optional[asyncio.StreamWriter] = None
-        self.have_connection: asyncio.Event = asyncio.Event()
-        self.port: Optional[int] = None
-
-    async def start_serving(
-        self,
-        host: str,
-    ) -> None:
-        server = await asyncio.start_server(self._handle_connection, host, 0)
-        (socket,) = server.sockets
-        self.port = socket.getsockname()[1]
-
-    async def _handle_connection(
-        self, reader: asyncio.StreamReader, writer: asyncio.StreamWriter
-    ) -> None:
-        self.reader = reader
-        self.writer = writer
-        self.have_connection.set()
-
-    async def wait_for_task_worker_connection(
-        self, timeout: float = 30
-    ) -> Tuple[asyncio.StreamReader, asyncio.StreamWriter]:
-        await asyncio.wait_for(self.have_connection.wait(), timeout=timeout)
-        assert self.reader is not None
-        assert self.writer is not None
-        return self.reader, self.writer
-
-    async def send_message(self, bs: bytes) -> None:
-        if not self.have_connection.is_set():
-            await self.wait_for_task_worker_connection()
-        assert self.writer is not None
-        msg_len = struct.pack(">i", len(bs))
-        self.writer.write(msg_len)
-        self.writer.write(bs)
-        await self.writer.drain()
-
-    async def receive_message(self) -> Any:
-        if not self.have_connection.is_set():
-            await self.wait_for_task_worker_connection()
-        assert self.reader is not None
-        (result_len,) = struct.unpack(">i", await self.reader.read(4))
-        result_bs = await self.reader.read(result_len)
-        return pickle.loads(result_bs)
-
-    async def close(self) -> None:
-        if self.writer:
-            self.writer.close()
-            await self.writer.wait_closed()
-            self.have_connection.clear()
-        if self.server:
-            self.server.close()
-            await self.server.wait_closed()
-
-
-async def agent_start_serving(
-    host: str,
-) -> AgentTaskWorkerServer:
-    server = AgentTaskWorkerServer()
-    await server.start_serving(host)
-    return server
-
-
 async def run_local(
     job: Job,
     cloud: Optional[Tuple[CloudProviderType, str]] = None,
@@ -1465,21 +1697,4 @@ class LocalHost(Host):
         )
 
     async def get_object_storage(self) -> ObjectStorage:
-        return LocalObjectStorage()
-
-
-async def _accumulate_stats(
-    get_worker_stats: Callable[[], Awaitable[Stats]], accumulator: StatsAccumulator
-) -> None:
-    """Modifies accumulator in place!"""
-    while True:
-        try:
-            stats = await get_worker_stats()
-            if stats.memory_in_use_gb > accumulator.max_memory_used_gb:
-                accumulator.max_memory_used_gb = stats.memory_in_use_gb
-        except asyncio.CancelledError:
-            raise
-        except Exception:
-            print("Error updating stats:\n" + traceback.format_exc())
-
-        await asyncio.sleep(1)
+        raise NotImplementedError("LocalHost has no object storage")
